@@ -1,5 +1,5 @@
 /* eslint node-core/documented-errors: "error" */
-/* eslint node-core/alphabetize-errors: "error" */
+/* eslint node-core/alphabetize-errors: ["error", {checkErrorDeclarations: true}] */
 /* eslint node-core/prefer-util-format-errors: "error" */
 
 'use strict';
@@ -12,7 +12,6 @@
 
 const {
   AggregateError,
-  ArrayFrom,
   ArrayIsArray,
   ArrayPrototypeFilter,
   ArrayPrototypeIncludes,
@@ -33,10 +32,10 @@ const {
   Number,
   NumberIsInteger,
   ObjectAssign,
-  ObjectDefineProperty,
   ObjectDefineProperties,
-  ObjectIsExtensible,
+  ObjectDefineProperty,
   ObjectGetOwnPropertyDescriptor,
+  ObjectIsExtensible,
   ObjectKeys,
   ObjectPrototypeHasOwnProperty,
   RangeError,
@@ -1113,7 +1112,11 @@ module.exports = {
 // Note: Node.js specific errors must begin with the prefix ERR_
 
 E('ERR_ACCESS_DENIED',
-  'Access to this API has been restricted. Permission: %s',
+  function(msg, permission = '', resource = '') {
+    this.permission = permission;
+    this.resource = resource;
+    return msg;
+  },
   Error);
 E('ERR_AMBIGUOUS_ARGUMENT', 'The "%s" argument is ambiguous. %s', TypeError);
 E('ERR_ARG_NOT_ITERABLE', '%s must be iterable', TypeError);
@@ -1205,12 +1208,12 @@ E('ERR_FEATURE_UNAVAILABLE_ON_PLATFORM',
   ', which is being used to run Node.js',
   TypeError);
 E('ERR_FS_CP_DIR_TO_NON_DIR',
-  'Cannot overwrite directory with non-directory', SystemError);
+  'Cannot overwrite non-directory with directory', SystemError);
 E('ERR_FS_CP_EEXIST', 'Target already exists', SystemError);
 E('ERR_FS_CP_EINVAL', 'Invalid src or dest', SystemError);
 E('ERR_FS_CP_FIFO_PIPE', 'Cannot copy a FIFO pipe', SystemError);
 E('ERR_FS_CP_NON_DIR_TO_DIR',
-  'Cannot overwrite non-directory with directory', SystemError);
+  'Cannot overwrite directory with non-directory', SystemError);
 E('ERR_FS_CP_SOCKET', 'Cannot copy a socket file', SystemError);
 E('ERR_FS_CP_SYMLINK_TO_SUBDIRECTORY',
   'Cannot overwrite symlink in subdirectory of self', SystemError);
@@ -1558,40 +1561,6 @@ E(
   ' `shortCircuit: true` in the hook\'s return.',
   Error,
 );
-E('ERR_MANIFEST_ASSERT_INTEGRITY',
-  (moduleURL, realIntegrities) => {
-    let msg = `The content of "${
-      moduleURL
-    }" does not match the expected integrity.`;
-    if (realIntegrities.size) {
-      const sri = ArrayPrototypeJoin(
-        ArrayFrom(realIntegrities.entries(),
-                  ({ 0: alg, 1: dgs }) => `${alg}-${dgs}`),
-        ' ',
-      );
-      msg += ` Integrities found are: ${sri}`;
-    } else {
-      msg += ' The resource was not found in the policy.';
-    }
-    return msg;
-  }, Error);
-E('ERR_MANIFEST_DEPENDENCY_MISSING',
-  'Manifest resource %s does not list %s as a dependency specifier for ' +
-  'conditions: %s',
-  Error);
-E('ERR_MANIFEST_INTEGRITY_MISMATCH',
-  'Manifest resource %s has multiple entries but integrity lists do not match',
-  SyntaxError);
-E('ERR_MANIFEST_INVALID_RESOURCE_FIELD',
-  'Manifest resource %s has invalid property value for %s',
-  TypeError);
-E('ERR_MANIFEST_INVALID_SPECIFIER',
-  'Manifest resource %s has invalid dependency mapping %s',
-  TypeError);
-E('ERR_MANIFEST_TDZ', 'Manifest initialization has not yet run', Error);
-E('ERR_MANIFEST_UNKNOWN_ONERROR',
-  'Manifest specified unknown error behavior "%s".',
-  SyntaxError);
 E('ERR_METHOD_NOT_IMPLEMENTED', 'The %s method is not implemented', Error);
 E('ERR_MISSING_ARGS',
   (...args) => {
@@ -1683,6 +1652,7 @@ E('ERR_PARSE_ARGS_UNKNOWN_OPTION', (option, allowPositionals) => {
 E('ERR_PERFORMANCE_INVALID_TIMESTAMP',
   '%d is not a valid timestamp', TypeError);
 E('ERR_PERFORMANCE_MEASURE_INVALID_OPTIONS', '%s', TypeError);
+E('ERR_REQUIRE_CYCLE_MODULE', '%s', Error);
 E('ERR_REQUIRE_ESM',
   function(filename, hasEsmSyntax, parentPath = null, packageJsonPath = null) {
     hideInternalStackFrames(this);
@@ -1898,6 +1868,10 @@ E('ERR_WORKER_INIT_FAILED', 'Worker initialization failure: %s', Error);
 E('ERR_WORKER_INVALID_EXEC_ARGV', (errors, msg = 'invalid execArgv flags') =>
   `Initiated Worker with ${msg}: ${ArrayPrototypeJoin(errors, ', ')}`,
   Error);
+E('ERR_WORKER_MESSAGING_ERRORED', 'The destination thread threw an error while processing the message', Error);
+E('ERR_WORKER_MESSAGING_FAILED', 'Cannot find the destination thread or listener', Error);
+E('ERR_WORKER_MESSAGING_SAME_THREAD', 'Cannot sent a message to the same thread', Error);
+E('ERR_WORKER_MESSAGING_TIMEOUT', 'Sending a message to another thread timed out', Error);
 E('ERR_WORKER_NOT_RUNNING', 'Worker instance not running', Error);
 E('ERR_WORKER_OUT_OF_MEMORY',
   'Worker terminated due to reaching memory limit: %s', Error);
