@@ -20,6 +20,8 @@ const { join, resolve, basename, isAbsolute } = require('path');
 
 const {
   kEmptyObject,
+  isWindows,
+  isMacOS,
 } = require('internal/util');
 const {
   validateFunction,
@@ -35,15 +37,30 @@ function lazyMinimatch() {
   return minimatch;
 }
 
-const isWindows = process.platform === 'win32';
-const isOSX = process.platform === 'darwin';
-
+/**
+ * @param {string} path
+ * @returns {Promise<DirentFromStats|null>}
+ */
 async function getDirent(path) {
-  return new DirentFromStats(basename(path), await lstat(path), path);
+  let stat;
+  try {
+    stat = await lstat(path);
+  } catch {
+    return null;
+  }
+  return new DirentFromStats(basename(path), stat, path);
 }
 
+/**
+ * @param {string} path
+ * @returns {DirentFromStats|null}
+ */
 function getDirentSync(path) {
-  return new DirentFromStats(basename(path), lstatSync(path), path);
+  const stat = lstatSync(path, { throwIfNoEntry: false });
+  if (stat === undefined) {
+    return null;
+  }
+  return new DirentFromStats(basename(path), stat, path);
 }
 
 class Cache {
@@ -56,7 +73,7 @@ class Cache {
     if (cached) {
       return cached;
     }
-    const promise = PromisePrototypeThen(getDirent(path), null, () => null);
+    const promise = getDirent(path);
     this.#statsCache.set(path, promise);
     return promise;
   }
@@ -65,12 +82,7 @@ class Cache {
     if (cached) {
       return cached;
     }
-    let val;
-    try {
-      val = getDirentSync(path);
-    } catch {
-      val = null;
-    }
+    const val = getDirentSync(path);
     this.#statsCache.set(path, val);
     return val;
   }
@@ -204,7 +216,7 @@ class Glob {
     }
     this.matchers = ArrayPrototypeMap(patterns, (pattern) => new (lazyMinimatch().Minimatch)(pattern, {
       __proto__: null,
-      nocase: isWindows || isOSX,
+      nocase: isWindows || isMacOS,
       windowsPathsNoEscape: true,
       nonegate: true,
       nocomment: true,
