@@ -2,8 +2,6 @@
 
 const {
   Date,
-  DatePrototypeGetMilliseconds,
-  DatePrototypeToUTCString,
   Symbol,
 } = primordials;
 
@@ -23,8 +21,8 @@ function utcDate() {
 
 function cache() {
   const d = new Date();
-  utcCache = DatePrototypeToUTCString(d);
-  setUnrefTimeout(resetCache, 1000 - DatePrototypeGetMilliseconds(d));
+  utcCache = d.toUTCString();
+  setUnrefTimeout(resetCache, 1000 - d.getMilliseconds());
 }
 
 function resetCache() {
