@@ -41,6 +41,7 @@ const {
 } = primordials;
 
 const EventEmitter = require('events');
+const { addAbortListener } = require('internal/events/abort_listener');
 const stream = require('stream');
 let debug = require('internal/util/debuglog').debuglog('net', (fn) => {
   debug = fn;
@@ -92,29 +93,29 @@ const {
   kBufferGen,
 } = require('internal/stream_base_commons');
 const {
+  ErrnoException,
+  ExceptionWithHostPort,
+  NodeAggregateError,
+  UVExceptionWithHostPort,
   codes: {
     ERR_INVALID_ADDRESS_FAMILY,
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
     ERR_INVALID_FD_TYPE,
-    ERR_INVALID_IP_ADDRESS,
     ERR_INVALID_HANDLE_TYPE,
+    ERR_INVALID_IP_ADDRESS,
+    ERR_MISSING_ARGS,
     ERR_SERVER_ALREADY_LISTEN,
     ERR_SERVER_NOT_RUNNING,
-    ERR_SOCKET_CONNECTION_TIMEOUT,
     ERR_SOCKET_CLOSED,
     ERR_SOCKET_CLOSED_BEFORE_CONNECTION,
-    ERR_MISSING_ARGS,
+    ERR_SOCKET_CONNECTION_TIMEOUT,
   },
-  ErrnoException,
-  ExceptionWithHostPort,
   genericNodeError,
-  NodeAggregateError,
-  UVExceptionWithHostPort,
 } = require('internal/errors');
 const { isUint8Array } = require('internal/util/types');
 const { queueMicrotask } = require('internal/process/task_queues');
-const { kEmptyObject, guessHandleType, promisify } = require('internal/util');
+const { kEmptyObject, guessHandleType, promisify, isWindows } = require('internal/util');
 const {
   validateAbortSignal,
   validateBoolean,
@@ -133,7 +134,7 @@ let dns;
 let BlockList;
 let SocketAddress;
 let autoSelectFamilyDefault = getOptionValue('--network-family-autoselection');
-let autoSelectFamilyAttemptTimeoutDefault = 250;
+let autoSelectFamilyAttemptTimeoutDefault = getOptionValue('--network-family-autoselection-attempt-timeout');
 
 const { clearTimeout, setTimeout } = require('timers');
 const { kTimeout } = require('internal/timers');
@@ -141,8 +142,6 @@ const { kTimeout } = require('internal/timers');
 const DEFAULT_IPV4_ADDR = '0.0.0.0';
 const DEFAULT_IPV6_ADDR = '::';
 
-const isWindows = process.platform === 'win32';
-
 const noop = () => {};
 
 const kPerfHooksNetConnectContext = Symbol('kPerfHooksNetConnectContext');
@@ -150,6 +149,7 @@ const kPerfHooksNetConnectContext = Symbol('kPerfHooksNetConnectContext');
 const dc = require('diagnostics_channel');
 const netClientSocketChannel = dc.channel('net.client.socket');
 const netServerSocketChannel = dc.channel('net.server.socket');
+const netServerListen = dc.tracingChannel('net.server.listen');
 
 const {
   hasObserver,
@@ -1631,7 +1631,7 @@ function addClientAbortSignalOption(self, options) {
     process.nextTick(onAbort);
   } else {
     process.nextTick(() => {
-      disposable = EventEmitter.addAbortListener(signal, onAbort);
+      disposable = addAbortListener(signal, onAbort);
     });
   }
 }
@@ -1723,7 +1723,7 @@ function addServerAbortSignalOption(self, options) {
   if (signal.aborted) {
     process.nextTick(onAborted);
   } else {
-    const disposable = EventEmitter.addAbortListener(signal, onAborted);
+    const disposable = addAbortListener(signal, onAborted);
     self.once('close', disposable[SymbolDispose]);
   }
 }
@@ -1878,6 +1878,11 @@ function setupListenHandle(address, port, addressType, backlog, fd, flags) {
 
     if (typeof rval === 'number') {
       const error = new UVExceptionWithHostPort(rval, 'listen', address, port);
+
+      if (netServerListen.hasSubscribers) {
+        netServerListen.error.publish({ server: this, error });
+      }
+
       process.nextTick(emitErrorNT, this, error);
       return;
     }
@@ -1897,6 +1902,11 @@ function setupListenHandle(address, port, addressType, backlog, fd, flags) {
     const ex = new UVExceptionWithHostPort(err, 'listen', address, port);
     this._handle.close();
     this._handle = null;
+
+    if (netServerListen.hasSubscribers) {
+      netServerListen.error.publish({ server: this, error: ex });
+    }
+
     defaultTriggerAsyncIdScope(this[async_id_symbol],
                                process.nextTick,
                                emitErrorNT,
@@ -1905,6 +1915,10 @@ function setupListenHandle(address, port, addressType, backlog, fd, flags) {
     return;
   }
 
+  if (netServerListen.hasSubscribers) {
+    netServerListen.asyncEnd.publish({ server: this });
+  }
+
   // Generate connection key, this should be unique to the connection
   this._connectionKey = addressType + ':' + address + ':' + port;
 
@@ -1969,7 +1983,11 @@ function listenInCluster(server, address, port, addressType,
       const ex = new ExceptionWithHostPort(err, 'bind', address, port);
       return server.emit('error', ex);
     }
-
+    // If there was a handle, just close it to avoid fd leak
+    // but it doesn't look like that's going to happen right now
+    if (server._handle) {
+      server._handle.close();
+    }
     // Reuse primary's server handle
     server._handle = handle;
     // _listen2 sets up the listened handle, it is still named like this
@@ -1988,6 +2006,10 @@ Server.prototype.listen = function(...args) {
     throw new ERR_SERVER_ALREADY_LISTEN();
   }
 
+  if (netServerListen.hasSubscribers) {
+    netServerListen.asyncStart.publish({ server: this, options });
+  }
+
   if (cb !== null) {
     this.once('listening', cb);
   }
@@ -1998,11 +2020,13 @@ Server.prototype.listen = function(...args) {
 
   options = options._handle || options.handle || options;
   const flags = getFlags(options.ipv6Only);
+  //  Refresh the id to make the previous call invalid
+  this._listeningId++;
   // (handle[, backlog][, cb]) where handle is an object with a handle
   if (options instanceof TCP) {
     this._handle = options;
     this[async_id_symbol] = this._handle.getAsyncId();
-    listenInCluster(this, null, -1, -1, backlogFromArgs);
+    listenInCluster(this, null, -1, -1, backlogFromArgs, undefined, true);
     return this;
   }
   addServerAbortSignalOption(this, options);
